/*
 * MPC Feedback Module 1 for ATOM (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <assert.h>

int proc_max=0;
int inst_max=0;

#define RegMax 32

long* RegAccess;
int*  RegAccessC;
int*  RegAccessRW;
char* AddrValue;

void MPCDummy(void)
{
  printf("MPCDummy!\n");
}

void MPCInit(int proc_max1, int inst_max1)
{
  long x;
  long x1;
  long y;

  proc_max=proc_max1;
  inst_max=inst_max1;

  x1=proc_max*inst_max;
  x=(sizeof(long)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb1): Allocating %u bytes of memory [RegAccess]...\n", x);

  RegAccess=(long *) calloc(sizeof(long), x1);

  if (RegAccess == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb1): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    RegAccess[y]=0;
  }

  x1=proc_max*inst_max;
  x=(sizeof(int)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb1): Allocating %u bytes of memory [RegAccessC] ...\n", x);

  RegAccessC=(int *) calloc(sizeof(int), x1);

  if (RegAccess == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb1): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    RegAccessC[y]=0;
  }

  x1=proc_max*inst_max;
  x=(sizeof(int)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb1): Allocating %u bytes of memory [RegAccessRW]...\n", x);

  RegAccessRW=(int *) calloc(sizeof(int), x1);

  if (RegAccessRW == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb1): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    RegAccessRW[y]=0;
  }

  x1=proc_max*inst_max*8;
  x=(sizeof(char)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb1): Allocating %u bytes of memory [AddrValue]...\n", x);

  AddrValue=(char *) calloc(sizeof(char), x1);

  if (AddrValue == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb1): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    AddrValue[y]=0;
  }

  printf("\nATOM (eos_mpc_fb1): Allocated successfully!\n\n",x);
}

void MPCFinish(void)
{
  FILE *f;
  int i=0;
  int j=0;
  int k=0;
  char fname[1024];

  long val=0;
  long val1=0;
  int q=0;

  fprintf(stderr, "\nATOM (eos_mpc_fb1): Saving collected data ...\n");

  for (i=0; i<proc_max; i++)
  {
    sprintf(fname, "_eos_mpc_atom_tmp5.%u", i);

    if ((f=fopen(fname,"wt"))==NULL)
    {
      fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb1): Can't write file!\n");
      exit(1);
    }

    for (k=0; k<inst_max; k++)
    {
      if (RegAccessC[i*inst_max+k]!=0)
      {
        val=0;
        val1=1;
        for (q=0; q<8; q++)
        {
          val+=val1*AddrValue[(i*inst_max+k)*8+q];
          val1*=256;
        }

        fprintf(f, "inst={%5u}, change={%3u}, addr={%20ld} (0x%016lx), rw={%1u}, value={%20ld} (0x%016lx)\n", 
                   k, 
                   RegAccessC[i*inst_max+k],
                   RegAccess[i*inst_max+k],
                   RegAccess[i*inst_max+k],
                   RegAccessRW[i*inst_max+k],
                   val,
                   val);
      }
      else
      {
        fprintf(f, "inst={%5u}, change={%3u}, addr={%20ld} (0x%016lx), rw={%1u}, value={%20ld} (0x%016lx)\n", 
                   k, 
                   RegAccessC[i*inst_max+k],
                   0,
                   0,
                   0,
                   0,
                   0);
      }
    }
   
    fclose(f);
  }

  fprintf(stderr, "ATOM (eos_mpc_fb1): Data saved successfully!\n");
}

void MPCRegAccess(long addr, int proc, int inst, int rw)
{
  int q=0;

  if (RegAccessC[proc*inst_max+inst]==2) return;
  else if (RegAccessC[proc*inst_max+inst]==1)
  {
    if (RegAccess[proc*inst_max+inst]!=addr)
    {
      RegAccessC[proc*inst_max+inst]=2;
    }
  }
  else
  {
    RegAccessC[proc*inst_max+inst]=1;
    RegAccess[proc*inst_max+inst]=addr;
    RegAccessRW[proc*inst_max+inst]=rw;

    if (rw==0)
    {
      for (q=0; q<8; q++) AddrValue[(proc*inst_max+inst)*8+q]=* (char*) (addr+q);
    }
  }
}
