// $Id: FlexLexer.h,v 1.2 1998/12/01 14:00:09 npt Exp $
// FlexLexer.h -- define classes for lexical analyzers generated by flex

// Copyright (c) 1993 The Regents of the University of California.
// All rights reserved.
//
// This code is derived from software contributed to Berkeley by
// Kent Williams and Tom Epperly.
//
// Redistribution and use in source and binary forms are permitted provided
// that: (1) source distributions retain this entire copyright notice and
// comment, and (2) distributions including binaries display the following
// acknowledgement:  ``This product includes software developed by the
// University of California, Berkeley and its contributors'' in the
// documentation or other materials provided with the distribution and in
// all advertising materials mentioning features or use of this software.
// Neither the name of the University nor the names of its contributors may
// be used to endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// This code was modified by N. Topham, University of Edinburgh, for inclusion
// in the Octave system. November 1996.

#ifndef __FLEX_LEXER_H
#define __FLEX_LEXER_H

// This defines YYSTYPE as class token_t so the translation unit that
// include this file and defines yylex must also define token_t

#ifndef YYSTYPE
#  define YYSTYPE token_t
#endif

class YYSTYPE ;

#define MAX_TOKEN_SIZE  256
#define STATE_STACK_SIZE 10

// This file defines two classes.  The first, FlexLexer, is an abstract
// class which specifies the external interface provided to flex C++
// lexer objects.  The second, yyFlexLexer, fills out most of the meat
// of the lexer class; its internals may vary from lexer to lexer
// depending on things like whether REJECT is used.
//
// If you want to create multiple lexer classes, you use the -P flag
// to rename each yyFlexLexer to some other xxFlexLexer.

#include <iostream.h>

extern "C++" {

struct yy_buffer_state;
typedef int yy_state_type;

class FlexLexer {
public:
	virtual ~FlexLexer()	{ }

	const char* YYText()	{ return yytext; }
	int YYLeng()		{ return yyleng; }

	virtual void
		yy_switch_to_buffer( struct yy_buffer_state* new_buffer ) = 0;
	virtual struct yy_buffer_state*
		yy_create_buffer( istream* s, int size ) = 0;
	virtual void yy_delete_buffer( struct yy_buffer_state* b ) = 0;
	virtual void yyrestart( istream* s ) = 0;

	virtual int yylex(YYSTYPE* lvalp) = 0;

        // Public data members added by Nigel Topham.
        //
	// Note: these should really be inherited from a base class,
	//       but flex as does not support this we have to modify
	//       what flex provides.

	int* lnum ;           // pointer to yylineno in derived class
	char errbuf [1000] ;  // buffer for complex lexer error messages
	
  // The following are context variables that are set as mid-rule
  // actions by the bison parser. They determine how the lexer
  // treats certain context-dependent character sequences.

	int expecting_operator ; // != 0 if "()" and "[]" should be 
                                 // recognised as operators.
	
        int in_template ;        // != 0 if parsing between "<" and ">" of
	                         // a template declaration.

protected:
	char* yytext;
	int yyleng;
};


class yyFlexLexer : public FlexLexer {
public:
	// arg_yyin and arg_yyout default to the cin and cout, but we
	// only make that assignment when initializing in yylex().
	yyFlexLexer( int* lno, int* nkw, istream* arg_yyin = 0, ostream* arg_yyout = 0 )
		{
		lnum = lptr = lno ;
		needkwd = nkw ;
		expecting_operator = in_template = 0 ;
 		yyin = arg_yyin;
		yyout = arg_yyout;
		yy_c_buf_p = 0;
		yy_init = 1;
		yy_start = 0;

		yy_did_buffer_switch_on_eof = 0;

		yy_looking_for_trail_begin = 0;
		yy_more_flag = 0;
		yy_more_len = 0;

		yy_start_stack_ptr = yy_start_stack_depth = 0;
		yy_start_stack = 0;

		yy_current_buffer = 0;

#ifdef YY_USES_REJECT
		yy_state_buf = new yy_state_type[YY_BUF_SIZE + 2];
#else
		yy_state_buf = 0;
#endif
		filename = 0 ;
		}

	virtual ~yyFlexLexer()
		{
		delete yy_state_buf;
		yy_delete_buffer (yy_current_buffer) ;
		}

	void yy_switch_to_buffer( struct yy_buffer_state* new_buffer );
	struct yy_buffer_state* yy_create_buffer( istream* s, int size );
	void yy_delete_buffer( struct yy_buffer_state* b );
	void yyrestart( istream* s );

	virtual int yylex(YYSTYPE*);
	virtual void setSourceFile(const char*, int, int) {}

	char* filename ;
	int inioctl ;

protected:
	virtual int LexerInput( char* buf, int max_size );
	virtual void LexerOutput( const char* buf, int size );
	virtual void LexerError( const char* msg );
	virtual void LexerInit () {}
	virtual const char* matchkeyword( const char*, int* ) = 0 ;
	virtual const char* matchdirective( const char*, int* ) = 0 ;

	void yyunput( int c, char* buf_ptr );
	int yyinput();

	void yy_load_buffer_state();
	void yy_init_buffer( struct yy_buffer_state* b, istream* s );

	int yy_start_stack_ptr;
	int yy_start_stack_depth;
	int* yy_start_stack;

	void yy_push_state( int new_state );
	void yy_pop_state();
	int yy_top_state();

	yy_state_type yy_get_previous_state();
	yy_state_type yy_try_NUL_trans( yy_state_type current_state );
	int yy_get_next_buffer();

	istream* yyin;	// input source for default LexerInput
	ostream* yyout;	// output sink for default LexerOutput

	struct yy_buffer_state* yy_current_buffer;

	// yy_hold_char holds the character lost when yytext is formed.
	char yy_hold_char;

	// Number of characters read into yy_ch_buf.
	int yy_n_chars;

	// Points to current character in buffer.
	char* yy_c_buf_p;

	int yy_init;		// whether we need to initialize
	int yy_start;		// start state number

	// Flag which is used to allow yywrap()'s to do buffer switches
	// instead of setting up a fresh yyin.  A bit of a hack ...
	int yy_did_buffer_switch_on_eof;

	// The following are not always needed, but may be depending
	// on use of certain flex features (like REJECT or yymore()).

	yy_state_type yy_last_accepting_state;
	char* yy_last_accepting_cpos;

	yy_state_type* yy_state_buf;
	yy_state_type* yy_state_ptr;

	char* yy_full_match;
	int* yy_full_state;
	int yy_full_lp;

	int yy_lp;
	int yy_looking_for_trail_begin;

	int yy_more_flag;
	int yy_more_len;

  // added by NPT for C++ lexer functionality

	void init_tokenBuffer () { tbp = 0 ; }
	void put_tokenBuffer (char c)  { tokenBuffer [tbp++] = c ; }
	void append_tokenBuffer (const char* s)
	{
	  while (*s)
	    tokenBuffer [tbp++] = *s++ ;
	}
        char* get_tokenBuffer () { put_tokenBuffer ('\0') ; return tokenBuffer ; }
        int token_length () { return tbp ; }
	void push_state (int s) { stateStack [ssp++] = s ; }
	int pop_state () { return stateStack [--ssp] ; }

        // pointer to yylineno (external to this object)
        // this pointer is synonymous with lnum
	int *lptr ;

        // pointer to needkwd (external to this object, used by Fortran)
	int *needkwd ;

private:
	char tokenBuffer [MAX_TOKEN_SIZE] ;
	int tbp ;
	int stateStack [STATE_STACK_SIZE] ;
	int ssp ; 
};

}

#endif
