/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) University of Edinburgh, 1993.                       |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

// $Id: Strings.cc,v 1.2 1999/09/20 13:09:24 npt Exp $

// Source for character String member functions.

#include "Octave/Base/Strings.h"
#include "Octave/Base/Errors.h"

string::string () {
  p = new srep ;
  p->s = 0 ;
  p->n = 1 ;
}

string::string (char *s) {
  p = new srep ;
  if (s)
    {
      p->s = new char[ strlen(s)+1 ] ;
      strcpy(p->s, s) ;
    }
  else
    p->s = 0 ;
  p->n = 1 ;
}

string::string (const char *s) {
  p = new srep ;
  if (s)
    {
      p->s = new char[ strlen(s)+1 ] ;
      strcpy(p->s, s) ;
    }
  else
    p->s = 0 ;
  p->n = 1 ;
}

string::string (string &x) {
  x.p->n++ ;
  p = x.p ;
}

string::string (string *sp) {
  if (sp) {
    sp->p->n++ ;
    p = sp->p ;
  }
  else {
    p = new srep ;
    p->s = 0 ;
    p->n = 1 ;
  }
}

string::~string () {
  if (p && --p->n == 0) {
    delete [] p->s ;
    delete p ;
  }
}

string& string::operator= (char *s) {
  if (p->n > 1) {
    p->n-- ;
    p = new srep ;
  }
  else if (p->n == 1)
    delete [] p->s ;

  p->s = new char[ strlen(s) + 1 ] ;
  strcpy(p->s, s) ;
  p->n = 1 ;
  return *this ;
}

string& string::operator= (const char *s) {
  if (p->n > 1) {
    p->n-- ;
    p = new srep ;
  }
  else if (p->n == 1)
    delete [] p->s ;

  p->s = new char[ strlen(s) + 1 ] ;
  strcpy(p->s, s) ;
  p->n = 1 ;
  return *this ;
}

string& string::operator= (string& x) {
  x.p->n++ ;
  if (--p->n == 0) {
    delete [] p->s ;
    delete p ;
  }
  
  p = x.p ;
  return *this ;
}

string& string::operator= (string* x) {
  x->p->n++ ;
  if (this && this->p->n == 1) {
    delete [] p->s ;
    delete p ;
  }
  
  p = x->p ;
  return *this ;
}

ostream& operator<< (ostream& s, string& x) {
  return s << x.p->s ;
}

istream& operator>> (istream& s, string& x) {
  char buffer[1024] ;
  s >> buffer ;
  x = buffer ;
  return s ;
}

char& string::operator[] (int i) {
  if (i < 0 || strlen(p->s) < i) 
  {
    err ("string index out of range") ;
    return p->s[strlen(p->s)] ;
  }
  else
    return p->s[i] ;
}

int operator== (string &x, char *s) {
  return strcmp(x(), s) == 0 ; }

int operator== (string &x, const char *s) {
  return strcmp(x(), s) == 0 ; }

int operator== (string &x, string &y) {
  return strcmp(x(), y()) == 0 ; }

int operator!= (string &x, char *s) {
  return strcmp(x(), s) != 0 ; }

int operator!= (string &x, string &y) {
  return strcmp(x(), y()) != 0 ; }

int operator!= (string &x, const char *s) {
  return strcmp(x(), s) != 0 ; }
