C ** Version modified for SPEC Benchmark suite - see Comments below
C ***
      program shalow
C     BENCHMARK WEATHER PREDICTION PROGRAM FOR COMPARING THE
C     PREFORMANCE OF CURRENT SUPERCOMPUTERS. THE MODEL IS
C     BASED OF THE PAPER - THE DYNAMICS OF FINITE-DIFFERENCE
C     MODELS OF THE SHALLOW-WATER EQUATIONS, BY ROBERT SADOURNY
C     J. ATM. SCIENCES, VOL 32, NO 4, APRIL 1975.
C
C     CODE BY PAUL N. SWARZTRAUBER, NATIONAL CENTER FOR
C     ATMOSPHERIC RESEARCH, BOULDER, CO,  OCTOBER 1984.
C
C     MODIFIED BY R. K. SATO, NCAR, APRIL 7, 1986 FOR MULTITASKING.
C     MODIFIED FOR SPEC to run longer: ITMAX inCremented from 120 to
C                                      1200 - J.Lo 7/19/90
C     Modified by Bodo Parady for the SPECpar suite.  Various
C     compilation sizes added.  Iterations reduced to orginal
C     problem, but size increased 4x in each dimension.
C
C     Further modified by Reinhold Weicker (Siemens Nixdorf) for the
C     SPEC CFP95 suite:
C     Changed back to the form with PARAMETER statements for
C     N1 and N2, set N1 = N2 = 1335.
C     The execution time is determined by these values and the
C     variable ITMAX (number of iterations) read from the input file.
C     Execution time is linear in ITMAX.
        implicit integer (i-n)
        implicit real*8 (a-h, o-z)
C      PARAMETER (N1=1061, N2=1061)
        parameter (n1=1335, n2=1335)
        real xyz(10, 10), chuk(30, 30, 20)
        integer abc, qwerty
        common // u(n1, n2), v(n1, n2), p(n1, n2), unew(n1, n2), vnew(n1
     +, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n2), cu(
     +n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
C
        write (unit=6, fmt=*) ' SPEC benchmark 171.swim'
        write (unit=6, fmt=*) ' '
        open (unit=7, FILE='SWIM7', STATUS='UNKNOWN') 
C       REQUEST PROCESSORS FOR MICROTASKING
C SPEC removed CMIC$ GETCPUS 4
C
C
C       INITIALIZE CONSTANTS AND ARRAYS
C
        call inital()
C
C     PRINT INITIAL VALUES
C
        write (unit=6, fmt=390) n, m, dx, dy, dt, alpha, itmax
  390   format (' NUMBER OF POINTS IN THE X DIRECTION',I8/' NUMBER OF PO
     +INTS IN THE Y DIRECTION',I8/' GRID SPACING IN THE X DIRECTION    '
     +,F8.0/' GRID SPACING IN THE Y DIRECTION    ',F8.0/' TIME STEP     
     +                     ',F8.0/' TIME FILTER PARAMETER              '
     +,F8.3/' NUMBER OF ITERATIONS               ',I8)
        mnmin = min0(m, n)
C initial data writes removed for SPEC
C      WRITE(6,391) (POLD(I,I),I=1,MNMIN)
C  391 FORMAT(/' INITIAL DIAGONAL ELEMENTS OF P ', //(8E15.7))
C      WRITE(6,392) (UOLD(I,I),I=1,MNMIN)
C  392 FORMAT(/' INITIAL DIAGONAL ELEMENTS OF U ', //(8E15.7))
C      WRITE(6,393) (VOLD(I,I),I=1,MNMIN)
C  393 FORMAT(/' INITIAL DIAGONAL ELEMENTS OF V ', //(8E15.7))
C        DETERMINE OVERHEAD OF TIMING CALLS
C  6/22/95 for SPEC: JWR: Initialization of TIME
        time = 0
        ncycle = 0
   90   ncycle = ncycle+1
C
C     COMPUTE CAPITAL  U, CAPITAL V, Z AND H
C
        call calc1()
C
C     COMPUTE NEW VALUES U,V AND P
C
        call calc2()
C
        time = time+dt
        if (mod(ncycle, mprint) .ne. 0) goto 370
        ptime = time/3600.
C
C *** modified for SPEC results verification
C *** We want to ensure that all calculations were done
C *** so we "use" all of the computed results.
C ***
C *** Since all of the comparisons of the individual diagnal terms
C *** often differ in the smaller values, the check we have selected
C *** is to add the absolute values of all terms and print these results
C
        write (unit=7, fmt=350) ncycle, ptime
  350   format (/' CYCLE NUMBER',I5,' MODEL TIME IN  HOURS',F6.2)
        write (unit=7, fmt=360) (unew(i, i), i=1,mnmin,10)
  360   format (/' DIAGONAL ELEMENTS OF U ',//(8E15.7))
C ***
        pcheck = 0.0D0
        ucheck = 0.0D0
        vcheck = 0.0D0
        do 3500 icheck = 1, mnmin
          do 4500 jcheck = 1, mnmin
            pcheck = pcheck+abs(pnew(icheck, jcheck))
            ucheck = ucheck+abs(unew(icheck, jcheck))
            vcheck = vcheck+abs(vnew(icheck, jcheck))
 4500       continue
          unew(icheck, icheck) = unew(icheck, icheck)*(mod(icheck, 100)/
     +100.)
 3500     continue
C ***
C ***
        write (unit=6, fmt=366) pcheck, ucheck, vcheck
  366   format (/,' Pcheck = ',E12.4,/,' Ucheck = ',E12.4,/,' Vcheck = '
     +,E12.4,/)
  370   continue
C        TEST FOR END OF RUN
        if (ncycle .ge. itmax) then
          stop
        end if
C
C     TIME SMOOTHING AND UPDATE FOR NEXT CYCLE
C
        if (ncycle .le. 1) then
          call calc3z()
        else
          call calc3()
        end if
C
        goto 90
      end
      subroutine inital()
C        INITIALIZE CONSTANTS AND ARRAYS
C           R. K. SATO 4/7/86
C
        implicit real*8 (a-h, o-z)
        parameter (n1=1335, n2=1335)
        common // u(n1, n2), v(n1, n2), p(n1, n2), unew(n1, n2), vnew(n1
     +, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n2), cu(
     +n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
C
C     NOTE BELOW THAT TWO DELTA T (TDT) IS SET TO DT ON THE FIRST
C     CYCLE AFTER WHICH IT IS RESET TO DT+DT.
C
C The following code  was in SWM256, however, it was replaced by 
C		READ statements to avoid calculations to be done during
C		compile time.
C
C      DT = 20.
C
C      DX = .25E5
C      DY = .25E5
C      A = 1.E6
C      ALPHA = .001
C      ITMAX = 1200
C      MPRINT = 1200
C      M = N1 - 1
C      N = N2 - 1
        read (unit=5, fmt=*) dt
        read (unit=5, fmt=*) dx
        read (unit=5, fmt=*) dy
        read (unit=5, fmt=*) a
        read (unit=5, fmt=*) alpha
        read (unit=5, fmt=*) itmax
        read (unit=5, fmt=*) mprint
        read (unit=5, fmt=*) m
        read (unit=5, fmt=*) n
        tdt = dt
        mp1 = m+1
        np1 = n+1
        el = n*dx
        pi = 4.D0*atan(1.D0)
        tpi = pi+pi
        di = tpi/m
        dj = tpi/n
        pcf = pi*pi*a*a/(el*el)
C
C     INITIAL VALUES OF THE STREAM FUNCTION AND P
C
        do 50 j = 1, np1
          do 50 i = 1, mp1
            psi(i, j) = a*sin((i-.5D0)*di)*sin((j-.5D0)*dj)
            p(i, j) = pcf*(cos(2.D0*(i-1)*di)+cos(2.D0*(j-1)*dj))+50000.
     +D0
   50       continue
C
C     INITIALIZE VELOCITIES
C
        do 60 j = 1, n
          do 60 i = 1, m
            u(i+1, j) = -(psi(i+1, j+1)-psi(i+1, j))/dy
            v(i, j+1) = (psi(i+1, j+1)-psi(i, j+1))/dx
   60       continue
C
C     PERIODIC CONTINUATION
C
        do 70 j = 1, n
          u(1, j) = u(m+1, j)
          v(m+1, j+1) = v(1, j+1)
   70     continue
        do 75 i = 1, m
          u(i+1, n+1) = u(i+1, 1)
          v(i, 1) = v(i, n+1)
   75     continue
        u(1, n+1) = u(m+1, 1)
        v(m+1, 1) = v(1, n+1)
        do 86 j = 1, np1
          do 86 i = 1, mp1
            uold(i, j) = u(i, j)
            vold(i, j) = v(i, j)
            pold(i, j) = p(i, j)
   86       continue
C        END OF INITIALIZATION
        return
      end
C SPEC removed CCMIC$ MICRO
      subroutine calc1()
C
C        COMPUTE CAPITAL  U, CAPITAL V, Z AND H
C
        implicit real*8 (a-h, o-z)
        parameter (n1=1335, n2=1335)
        common // u(n1, n2), v(n1, n2), p(n1, n2, 10, 5), unew(n1, n2), 
     +vnew(n1, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n
     +2), cu(n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
C
        fsdx = 4.D0/dx
        fsdy = 4.D0/dy
C SPEC removed CCMIC$ DO GLOBAL
        do 100 j = 1, n
          do 100 i = 1, m
            cu(i+1, j) = .5D0*(p(i+1, j, 1, 2)+p(i, j, 1, 2))*u(i+1, j)
            cv(i, j+1) = .5D0*(p(i, j+1, 1, 2)+p(i, j, 1, 2))*v(i, j+1)
            z(i+1, j+1) = (fsdx*(v(i+1, j+1)-v(i, j+1))-fsdy*(u(i+1, j+1
     +)-u(i+1, j)))/(p(i, j, 1, 2)+p(i+1, j, 1, 2)+p(i+1, j+1, 1, 2)+p(i
     +, j+1, 1, 2))
            h(i, j) = p(i, j, 1, 2)+.25D0*(u(i+1, j)*u(i+1, j)+u(i, j)*u
     +(i, j)+v(i, j+1)*v(i, j+1)+v(i, j)*v(i, j))
  100       continue
C
C     PERIODIC CONTINUATION
C
        do 110 j = 1, n
          cu(1, j) = cu(m+1, j)
          cv(m+1, j+1) = cv(1, j+1)
          z(1, j+1) = z(m+1, j+1)
          h(m+1, j) = h(1, j)
  110     continue
        do 115 i = 1, m
          cu(i+1, n+1) = cu(i+1, 1)
          cv(i, 1) = cv(i, n+1)
          z(i+1, 1) = z(i+1, n+1)
          h(i, n+1) = h(i, 1)
  115     continue
        cu(1, n+1) = cu(m+1, 1)
        cv(m+1, 1) = cv(1, n+1)
        z(1, 1) = z(m+1, n+1)
        h(m+1, n+1) = h(1, 1)
C        END OF LOOP 100 CALCULATIONS
        return
      end
C SPEC removed CCMIC$ MICRO
      subroutine calc2()
C
C        COMPUTE NEW VALUES OF U,V,P
C
        implicit real*8 (a-h, o-z)
        parameter (n1=1335, n2=1335)
        common // u(n1, n2), v(n1, n2), p(n1, n2), unew(n1, n2), vnew(n1
     +, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n2), cu(
     +n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
        tdts8 = tdt/8.D0
        tdtsdx = tdt/dx
        tdtsdy = tdt/dy
C SPEC removed CCMIC$ DO GLOBAL
        do 200 j = 1, n
          do 200 i = 1, m
            unew(i+1, j) = uold(i+1, j)+tdts8*(z(i+1, j+1)+z(i+1, j))*(c
     +v(i+1, j+1)+cv(i, j+1)+cv(i, j)+cv(i+1, j))-tdtsdx*(h(i+1, j)-h(i,
     + j))
            vnew(i, j+1) = vold(i, j+1)-tdts8*(z(i+1, j+1)+z(i, j+1))*(c
     +u(i+1, j+1)+cu(i, j+1)+cu(i, j)+cu(i+1, j))-tdtsdy*(h(i, j+1)-h(i,
     + j))
            pnew(i, j) = pold(i, j)-tdtsdx*(cu(i+1, j)-cu(i, j))-tdtsdy*
     +(cv(i, j+1)-cv(i, j))
  200       continue
C
C     PERIODIC CONTINUATION
C
        do 210 j = 1, n
          unew(1, j) = unew(m+1, j)
          vnew(m+1, j+1) = vnew(1, j+1)
          pnew(m+1, j) = pnew(1, j)
  210     continue
        do 215 i = 1, m
          unew(i+1, n+1) = unew(i+1, 1)
          vnew(i, 1) = vnew(i, n+1)
          pnew(i, n+1) = pnew(i, 1)
  215     continue
        unew(1, n+1) = unew(m+1, 1)
        vnew(m+1, 1) = vnew(1, n+1)
        pnew(m+1, n+1) = pnew(1, 1)
C
        return
      end
      subroutine calc3z()
C
C         TIME SMOOTHER FOR FIRST ITERATION
C
        implicit real*8 (a-h, o-z)
        parameter (n1=1335, n2=1335)
        common // u(n1, n2), v(n1, n2), p(n1, n2), unew(n1, n2), vnew(n1
     +, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n2), cu(
     +n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
C
        tdt = tdt+tdt
        do 400 j = 1, np1
          do 400 i = 1, mp1
            uold(i, j) = u(i, j)
            vold(i, j) = v(i, j)
            pold(i, j) = p(i, j)
            u(i, j) = unew(i, j)
            v(i, j) = vnew(i, j)
            p(i, j) = pnew(i, j)
  400       continue
        return
      end
C SPEC removed CCMIC$ MICRO
      subroutine calc3()
C
C         TIME SMOOTHER
C
        implicit real*8 (a-h, o-z)
        parameter (n1=1335, n2=1335)
        common // u(n1, n2), v(n1, n2), p(n1, n2), unew(n1, n2), vnew(n1
     +, n2), pnew(n1, n2), uold(n1, n2), vold(n1, n2), pold(n1, n2), cu(
     +n1, n2), cv(n1, n2), z(n1, n2), h(n1, n2), psi(n1, n2)
C
        common /cons/ dt, tdt, dx, dy, a, alpha, itmax, mprint, m, n, mp
     +1, np1, el, pi, tpi, di, dj, pcf
C
C        TIME SMOOTHING AND UPDATE FOR NEXT CYCLE
C
C SPEC removed CCMIC$ DO GLOBAL
        do 300 j = 1, n
          do 300 i = 1, m
            uold(i, j) = u(i, j)+alpha*(unew(i, j)-2.*u(i, j)+uold(i, j)
     +)
            vold(i, j) = v(i, j)+alpha*(vnew(i, j)-2.*v(i, j)+vold(i, j)
     +)
            pold(i, j) = p(i, j)+alpha*(pnew(i, j)-2.*p(i, j)+pold(i, j)
     +)
            u(i, j) = unew(i, j)
            v(i, j) = vnew(i, j)
            p(i, j) = pnew(i, j)
  300       continue
C
C     PERIODIC CONTINUATION
C
        do 320 j = 1, n
          uold(m+1, j) = uold(1, j)
          vold(m+1, j) = vold(1, j)
          pold(m+1, j) = pold(1, j)
          u(m+1, j) = u(1, j)
          v(m+1, j) = v(1, j)
          p(m+1, j) = p(1, j)
  320     continue
        do 325 i = 1, m
          uold(i, n+1) = uold(i, 1)
          vold(i, n+1) = vold(i, 1)
          pold(i, n+1) = pold(i, 1)
          u(i, n+1) = u(i, 1)
          v(i, n+1) = v(i, 1)
          p(i, n+1) = p(i, 1)
  325     continue
        uold(m+1, n+1) = uold(1, 1)
        vold(m+1, n+1) = vold(1, 1)
        pold(m+1, n+1) = pold(1, 1)
        u(m+1, n+1) = u(1, 1)
        v(m+1, n+1) = v(1, 1)
        p(m+1, n+1) = p(1, 1)
C
        return
      end
