/*
 * EOS Timers (Loop or other level) Windows (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>

#define timers_max $EOS1$
#define queue_max  $EOS2$

static int    first_run=0;

static double time_period=$EOS3$;
static double time_cur=0;
static int    queue=0;
static long   queue_total=0;

static double Timers[queue_max+1][timers_max+1];
static double Timers1[queue_max+1][timers_max+1];
static double Timers2[queue_max+1];
static long   Calls[queue_max+1][timers_max+1];

void eos_timers_init(void)
{
  long y;
  int q;
  FILE *f;

  for (q=0; q<queue_max; q++)
  {
    for (y=1; y<=timers_max; y++)
    {
      Timers[q][y]=0;
      Timers1[q][y]=0;
      Calls[q][y]=0;
    }
    Timers2[q]=0;
  }

  if ((f=fopen("_eos_llprof_out","wt"))==NULL)
  {
    printf("\nFatal Error (FATOM): Can't write file!\n");
    exit(1);
  }
  fclose(f);
}

void new_time(int queue)
{
  int i=0;

  Timers2[queue]=0;
  for (i=1; i<=timers_max; i++)
  {
    Timers1[queue][i]=(double) clock() / CLOCKS_PER_SEC;
  }
}

void save_timers(void)
{
  FILE *f;
  int i=0;

  unsigned int val=0;
  int q1=0;

  double time_all=0;

  if ((f=fopen("_eos_llprof_out","at"))==NULL)
  {
    printf("\nFatal Error (FATOM): Can't write file!\n");
    exit(1);
  }
         
  for (q1=0; q1<queue; q1++)
  {
    fprintf(f, "step={%8u}, ", queue_total);
    queue_total++;

    fprintf(f, "period={%5.2f}, ", Timers2[q1]);

    for (i=1; i<=timers_max; i++)
    {
      fprintf(f, " {%5.2f}", Timers[q1][i]);
      time_all+=Timers[q1][i];

      Timers[q1][i]=0;
      Timers1[q1][i]=0;
      Calls[q1][i]=0;
    }
  
    fprintf(f, "\n");
  }
   
  fclose(f);

  queue=0;
  time_cur=(double) clock() / CLOCKS_PER_SEC;
  new_time(queue);
}

void eos_timers_finish(void)
{
  save_timers();
  
  return;
}

void EOS_TIMER1(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
    queue=0;
    queue_total=0;
    new_time(queue);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[queue][tmr]++;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers1[queue][tmr]=tt;

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void EOS_TIMER2(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  tmr=*tmr1;
  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers[queue][tmr]+=tt - Timers1[queue][tmr];

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer1(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
    queue=0;
    queue_total=0;
    new_time(queue);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[queue][tmr]++;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers1[queue][tmr]=tt;

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer2(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  tmr=*tmr1;
  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers[queue][tmr]+=tt - Timers1[queue][tmr];

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer1_(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
    queue=0;
    queue_total=0;
    new_time(queue);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[queue][tmr]++;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers1[queue][tmr]=tt;

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer2_(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  tmr=*tmr1;
  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers[queue][tmr]+=tt - Timers1[queue][tmr];

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer1__(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
    queue=0;
    queue_total=0;
    new_time(queue);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[queue][tmr]++;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers1[queue][tmr]=tt;

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void eos_timer2__(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  tmr=*tmr1;
  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers[queue][tmr]+=tt - Timers1[queue][tmr];

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void _Z10eos_timer1i(int tmr)
{
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
    queue=0;
    queue_total=0;
    new_time(queue);
    first_run=1;
  }
  
  Calls[queue][tmr]++;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers1[queue][tmr]=tt;

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}

void _Z10eos_timer2i(int tmr)
{
  double tt=0;
  double tt1=0;

  tt=(double) clock() / CLOCKS_PER_SEC;
  Timers[queue][tmr]+=tt - Timers1[queue][tmr];

  tt1=tt-time_cur;
  if (tt1>time_period) 
  {
    Timers2[queue]=tt1;
    queue++;
    new_time(queue);
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }

  if (queue>queue_max)
  {
    save_timers();
    time_cur=(double) clock() / CLOCKS_PER_SEC;
  }
}
                 
