/*
 * MPC Feedback Module 3 for ATOM (instrumentation)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <machine/inst.h>
#include <cmplrs/atom.inst.h>

void InstrumentInit(int p1, char **p2) 
{
  AddCallProto("MPCInit(int)");
  AddCallProto("MPCRegAccess1(int, int, REGV)");
  AddCallProto("MPCRegAccess2(int, int, FREGV)");
}

void fggstrlwr(char* addr)
{
  char c;

  while ((c=*addr)!=0)
  {
    *addr=tolower(c);
    addr++;
  }
}

Instrument(int argc, char **argv, Obj *obj)
{
  Proc *p;  
  Block *b; 
  Inst *i;

  long rg1;
  long rg2;
  int  rg_type;

  FILE *fmpc1;

  char dummy[1024];
  char procname[1024];
  char procname1[1024];
  char addr1[16];
  char addr2[16];

  char ops[1024];
  char fr[2];

  int line=0;
  int l1=0;
  int l2=0;

  int ret=0;
  int ret1=0;

  int ifound=0;
  int imd=0;
  int ira=0;
  int irb=0;
  int irc=0;
  int iop=0;
  int rw=0;

  int proc=0;
  int inst_max=0;
  int inst_max1=0;

  int k=0;
  int j=0;

  int found1=0;

  /*************************** Main ***************************/
  printf("EOS MPC Feedback Module 3   (C) by Grigori G. Fursin   August,2001\n");

  if ((fmpc1=fopen("_eos_mpc_info2","rt"))==NULL)
  {
    fprintf(stderr, "\nFatal Error: Can't find file with parameters!\n");
    exit(1);
  }

  do
  {
    ret=fscanf(fmpc1, "%s", &dummy[0]);

    if (ret!=EOF && dummy[0]=='+')
    {
      ret=fscanf(fmpc1, "%s %s", &addr1[0], &addr2[0]);

      k=1;
      j=0;
      for (;k<strlen(dummy) && dummy[k]!=' '; k++) procname[j++]=dummy[k];
      procname[j]=0;

      fggstrlwr(procname);

      strcat(procname, "_tr_");

      printf("\nSearching subroutine: %s (%s - %s)\n", procname, addr1, addr2);

      l1=atoi(addr1);
      l2=atoi(addr2);

      for (p=GetFirstObjProc(obj); p!=NULL; p=GetNextProc(p))
      {
        if (ProcName(p)!=NULL) strcpy(procname1, ProcName(p));
        else strcpy(procname1, "");

        fggstrlwr(procname1);

        if (strcmp(procname1, procname)==0)
        {   
          printf("Subroutine %s found!\n", procname);

          inst_max=0;

          for (b=GetFirstBlock(p); b!=NULL; b=GetNextBlock(b))
          {
            for (i=GetFirstInst(b); i!=NULL; i=GetNextInst(i))
            {
              line=InstLineNo(i);

              if ((l1<=line) && (line<=l2))
              {
                found1=0;

                if ((GetInstClass(i) == ClassLoad))
                {
                  found1=1;
                  strcpy(fr, "");
                }
                else if ((GetInstClass(i) == ClassFload))
                {
                  found1=2;
                  strcpy(fr, "f");
                }
                else if ((GetInstClass(i) == ClassStore))
                {
                  found1=3;
                  strcpy(fr, "");
                }
                else if ((GetInstClass(i) == ClassFstore))
                {
                  found1=4;
                  strcpy(fr, "f");
                }

                if (found1!=0)
                {
                  imd=GetInstInfo(i, InstMemDisp);
                  ira=GetInstInfo(i, InstRA);
                  irb=GetInstInfo(i, InstRB);
                  irc=GetInstInfo(i, InstRC);
                  iop=GetInstInfo(i, InstOpcode);

                  switch(iop)
                  {
                    case op_lda:   {strcpy(ops, "lda");     rw=0; break;};
                    case op_ldah:  {strcpy(ops, "ldah");    rw=0; break;};
                    case op_ldbu:  {strcpy(ops, "ldbu");    rw=0; break;};
                    case op_ldq_u: {strcpy(ops, "ldq_u");   rw=0; break;;}
                    case op_ldwu:  {strcpy(ops, "ldwu");    rw=0; break;};
                    case op_stw:   {strcpy(ops, "stw");     rw=1; break;};
                    case op_stb:   {strcpy(ops, "stb");     rw=1; break;};
                    case op_stq_u: {strcpy(ops, "stq_u");   rw=1; break;};
                    case op_ldf:   {strcpy(ops, "ldf");     rw=0; break;};
                    case op_ldg:   {strcpy(ops, "ldg");     rw=0; break;};
                    case op_lds:   {strcpy(ops, "lds");     rw=0; break;};
                    case op_ldt:   {strcpy(ops, "ldt");     rw=0; break;};
                    case op_stf:   {strcpy(ops, "stf");     rw=1; break;};
                    case op_stg:   {strcpy(ops, "stg");     rw=1; break;};
                    case op_sts:   {strcpy(ops, "sts");     rw=1; break;};
                    case op_stt:   {strcpy(ops, "stt");     rw=1; break;};
                    case op_ldl:   {strcpy(ops, "ldl");     rw=0; break;};
                    case op_ldq:   {strcpy(ops, "ldq");     rw=0; break;};
                    case op_ldl_l: {strcpy(ops, "ldl_l");   rw=0; break;};
                    case op_ldq_l: {strcpy(ops, "ldq_l");   rw=0; break;};
                    case op_stl:   {strcpy(ops, "stl");     rw=1; break;};
                    case op_stq:   {strcpy(ops, "stq");     rw=1; break;};
                    case op_stl_c: {strcpy(ops, "stl_c");   rw=1; break;};
                    case op_stq_c: {strcpy(ops, "stq_c");   rw=1; break;};
    
                    default:       {strcpy(ops, "unknown"); rw=0; break;};
                  }

                  if (irb<30)
                  {                 
                    if (found1!=2 && found1!=3) 
                    {
                      switch(ira)
                      {
                        case  0: {rg1=REG_0 ;break;};
                        case  1: {rg1=REG_1 ;break;};
                        case  2: {rg1=REG_2 ;break;};
                        case  3: {rg1=REG_3 ;break;};
                        case  4: {rg1=REG_4 ;break;};
                        case  5: {rg1=REG_5 ;break;};
                        case  6: {rg1=REG_6 ;break;};
                        case  7: {rg1=REG_7 ;break;};
                        case  8: {rg1=REG_8 ;break;};
                        case  9: {rg1=REG_9 ;break;};
                        case 10: {rg1=REG_10;break;};
                        case 11: {rg1=REG_11;break;};
                        case 12: {rg1=REG_12;break;};
                        case 13: {rg1=REG_13;break;};
                        case 14: {rg1=REG_14;break;};
                        case 15: {rg1=REG_15;break;};
                        case 16: {rg1=REG_16;break;};
                        case 17: {rg1=REG_17;break;};
                        case 18: {rg1=REG_18;break;};
                        case 19: {rg1=REG_19;break;};
                        case 20: {rg1=REG_20;break;};
                        case 21: {rg1=REG_21;break;};
                        case 22: {rg1=REG_22;break;};
                        case 23: {rg1=REG_23;break;};
                        case 24: {rg1=REG_24;break;};
                        case 25: {rg1=REG_25;break;};
                        case 26: {rg1=REG_26;break;};
                        case 27: {rg1=REG_27;break;};
                        case 28: {rg1=REG_28;break;};
                        case 29: {rg1=REG_29;break;};
                        case 30: {rg1=REG_30;break;};
                        case 31: {rg1=REG_31;break;};
                      }
                      AddCallInst(i, InstBefore, "MPCRegAccess1", proc, inst_max, rg1);
                    }
                    else
                    {
                      switch(ira)
                      {
                        case  0: {rg2=FREG_0 ;break;};
                        case  1: {rg2=FREG_1 ;break;};
                        case  2: {rg2=FREG_2 ;break;};
                        case  3: {rg2=FREG_3 ;break;};
                        case  4: {rg2=FREG_4 ;break;};
                        case  5: {rg2=FREG_5 ;break;};
                        case  6: {rg2=FREG_6 ;break;};
                        case  7: {rg2=FREG_7 ;break;};
                        case  8: {rg2=FREG_8 ;break;};
                        case  9: {rg2=FREG_9 ;break;};
                        case 10: {rg2=FREG_10;break;};
                        case 11: {rg2=FREG_11;break;};
                        case 12: {rg2=FREG_12;break;};
                        case 13: {rg2=FREG_13;break;};
                        case 14: {rg2=FREG_14;break;};
                        case 15: {rg2=FREG_15;break;};
                        case 16: {rg2=FREG_16;break;};
                        case 17: {rg2=FREG_17;break;};
                        case 18: {rg2=FREG_18;break;};
                        case 19: {rg2=FREG_19;break;};
                        case 20: {rg2=FREG_20;break;};
                        case 21: {rg2=FREG_21;break;};
                        case 22: {rg2=FREG_22;break;};
                        case 23: {rg2=FREG_23;break;};
                        case 24: {rg2=FREG_24;break;};
                        case 25: {rg2=FREG_25;break;};
                        case 26: {rg2=FREG_26;break;};
                        case 27: {rg2=FREG_27;break;};
                        case 28: {rg2=FREG_28;break;};
                        case 29: {rg2=FREG_29;break;};
                        case 30: {rg2=FREG_30;break;};
                        case 31: {rg2=FREG_31;break;};
                      }

                      AddCallInst(i, InstBefore, "MPCRegAccess2", proc, inst_max, rg2);
                    }
                    inst_max++;
                  }
                }
              }
            }
          }

          if (inst_max>inst_max1) inst_max1=inst_max;
        }
      }  

      proc++;
    }
  } while (ret!=EOF);

  /************************************************************/
  printf("\nEbedding additional code ...\n");

  AddCallProgram(ProgramBefore,"MPCInit", proc);

  printf("\nInstrumentation finished!\n");

  fclose(fmpc1);
}
